/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.readers.its.bridge;

// for logging and error handling
import com.rempl.api.Logger;
import com.rempl.api.RuntimeProblem;

/**
 * Abstract Factory of bridges.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: BridgeFactory.java 1306 2011-05-02 16:04:25Z guard $
 */
public final class BridgeFactory {

    /**
     * Public singleton instance.
     */
    public static final BridgeFactory INSTANCE = new BridgeFactory();

    /**
     * Private ctor to avoid direct instantiation of the class.
     */
    private BridgeFactory() {
        // intentionally empty
    }

    /**
     * Instantiate a new bridge by its class name.
     * @param name The name of the class
     * @return The bridge
     */
    public Bridge makeBridge(final String name) {
        final Object bridge = this.instantiate(name);
        if (!(bridge instanceof Bridge)) {
            throw RuntimeProblem.make(
                "Invalid Bridge class %s",
                bridge.getClass().getName()
            );
        }
        Logger.debug(
            this,
            "#makeBridge('%s'): instantiated",
            name
        );
        return (Bridge) bridge;
    }

    /**
     * Instantiate a new builder by its class name.
     * @param name The name of the class
     * @return The builder
     */
    public CodeUrlBuilder makeCodeUrlBuilder(final String name) {
        final Object builder = this.instantiate(name);
        if (!(builder instanceof CodeUrlBuilder)) {
            throw RuntimeProblem.make(
                "Invalid CodeUrlBuilder class %s",
                builder.getClass().getName()
            );
        }
        Logger.debug(
            this,
            "#makeCodeUrlBuilder('%s'): instantiated",
            name
        );
        return (CodeUrlBuilder) builder;
    }

    /**
     * Instantiate a new builder by its class name.
     * @param name The name of the class
     * @return The builder
     */
    public IssueUrlBuilder makeIssueUrlBuilder(final String name) {
        final Object builder = this.instantiate(name);
        if (!(builder instanceof IssueUrlBuilder)) {
            throw RuntimeProblem.make(
                "Invalid IssueUrlBuilder class %s",
                builder.getClass().getName()
            );
        }
        Logger.debug(
            this,
            "#makeIssueUrlBuilder('%s'): instantiated",
            name
        );
        return (IssueUrlBuilder) builder;
    }

    /**
     * Instantiate a new builder by its class name.
     * @param name The name of the class
     * @return The builder
     */
    private Object instantiate(final String name) {
        Logger.debug(
            this,
            "#instantiate('%s'): trying to instantiate...",
            name
        );
        RuntimeException failure = null;
        Object obj = null;
        try {
            obj = this.getClass().forName(name).newInstance();
        } catch (ClassNotFoundException ex) {
            failure = RuntimeProblem.make(ex);
        } catch (InstantiationException ex) {
            failure = RuntimeProblem.make(ex);
        } catch (IllegalAccessException ex) {
            failure = RuntimeProblem.make(ex);
        }
        if (failure != null) {
            Logger.error(
                this,
                "#instantiate('%s'): %s",
                name,
                failure.getMessage()
            );
            throw failure;
        }
        Logger.info(
            this,
            "#instantiate('%s'): success",
            name
        );
        return obj;
    }

}
